"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.genSaltedPassword = genSaltedPassword;
exports.checkSaltedPassword = checkSaltedPassword;
exports.mkdirp = mkdirp;
Object.defineProperty(exports, "AWS", {
  enumerable: true,
  get: function () {
    return _awsSdk.default;
  }
});
Object.defineProperty(exports, "Knex", {
  enumerable: true,
  get: function () {
    return _knex.default;
  }
});
Object.defineProperty(exports, "moment", {
  enumerable: true,
  get: function () {
    return _moment.default;
  }
});
Object.defineProperty(exports, "mysql", {
  enumerable: true,
  get: function () {
    return _mysql.default;
  }
});
Object.defineProperty(exports, "ensure", {
  enumerable: true,
  get: function () {
    return _odJs.ensure;
  }
});
Object.defineProperty(exports, "NodeUtils", {
  enumerable: true,
  get: function () {
    return _csv.NodeUtils;
  }
});
Object.defineProperty(exports, "ODApp", {
  enumerable: true,
  get: function () {
    return _express.default;
  }
});
Object.defineProperty(exports, "handlerMaker", {
  enumerable: true,
  get: function () {
    return _handler.handlerMaker;
  }
});
Object.defineProperty(exports, "getDupKeyErrorIndexName", {
  enumerable: true,
  get: function () {
    return _mysql2.getDupKeyErrorIndexName;
  }
});
Object.defineProperty(exports, "isDeadLockError", {
  enumerable: true,
  get: function () {
    return _mysql2.isDeadLockError;
  }
});
Object.defineProperty(exports, "isDupKeyError", {
  enumerable: true,
  get: function () {
    return _mysql2.isDupKeyError;
  }
});
Object.defineProperty(exports, "isForeignKeyError", {
  enumerable: true,
  get: function () {
    return _mysql2.isForeignKeyError;
  }
});
Object.defineProperty(exports, "upsertSQL", {
  enumerable: true,
  get: function () {
    return _mysql2.upsertSQL;
  }
});
Object.defineProperty(exports, "gmailSender", {
  enumerable: true,
  get: function () {
    return _sendmail.gmailSender;
  }
});
Object.defineProperty(exports, "SlackNotifier", {
  enumerable: true,
  get: function () {
    return _slack.SlackNotifier;
  }
});
Object.defineProperty(exports, "storageDefiner", {
  enumerable: true,
  get: function () {
    return _definition.storageDefiner;
  }
});
exports.jwtUtil = exports.knex = exports.default = void 0;

var _bluebird = _interopRequireDefault(require("bluebird"));

var _awsSdk = _interopRequireDefault(require("aws-sdk"));

var _bcrypt = _interopRequireDefault(require("bcrypt"));

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _knex = _interopRequireDefault(require("knex"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _moment = _interopRequireDefault(require("moment"));

var _mysql = _interopRequireDefault(require("mysql"));

var _odJs = require("od-js");

var _csv = require("./csv");

var _express = _interopRequireDefault(require("./express"));

var _handler = require("./handler");

var _mysql2 = require("./mysql");

var _sendmail = require("./sendmail");

var _slack = require("./slack");

var _definition = require("./storage/definition");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-param-reassign */
const knex = (0, _knex.default)({
  client: 'mysql'
}); // use only for query-builder

exports.knex = knex;
let getMySQLConnection;
/**
 * Returns MySQL connection, and inject promise-version of query function.
 *
 * @param [options] {Object} if omitted, try to read from environment variable.
 *
 * {
 *   host: 'gameberrycashbackdb.cgta6qz5w1ey.us-east-1.rds.amazonaws.com',
 *   user: 'user',
 *   password: 'password',
 *   port: 3306,
 *   database: 'gcb',
 *   debug: false
 * }
 * @return {Connection}
 */

function getMySQLConnectionLambda(options) {
  if (options === undefined || options === null) {
    if (getMySQLConnection.options) {
      // eslint-disable-next-line prefer-destructuring
      options = getMySQLConnection.options;
    } else {
      options = {
        host: process.env.mysql_host,
        user: process.env.mysql_user,
        password: process.env.mysql_password,
        port: parseInt(process.env.mysql_port, 10),
        database: process.env.mysql_database,
        debug: parseInt(process.env.mysql_debug, 10) === 1
      };
    }
  }

  const conn = _mysql.default.createConnection(options);

  const orgQuery = conn.query.bind(conn);

  conn.query = function query(query) {
    return new _bluebird.default((resolve, reject) => {
      orgQuery(query, (err, res) => {
        err ? reject(err) : resolve(res);
      });
    });
  };

  return conn;
}
/**
 * Inject options before calling getMySQLConnection.
 *
 * @param options {object}
 */


getMySQLConnectionLambda.setOptions = function setOptions(options) {
  getMySQLConnection.options = options;
};
/**
 * bcryptjs 를 이용하여 salted password 를 생성한다.
 *
 * @param pw {string}
 * @param iteration {number=10}
 * @return {Promise}
 */


async function genSaltedPassword(pw, iteration = 10) {
  return new _bluebird.default((resolve, reject) => {
    _bcrypt.default.genSalt(iteration, (err, salt) => {
      if (err) {
        reject(err);
      } else {
        _bcrypt.default.hash(pw, salt, (err, salted) => {
          if (err) {
            reject(err);
          } else {
            resolve(salted);
          }
        });
      }
    });
  });
}
/**
 * Check if given {pw} matches salted password {salted}
 * @param pw {string}
 * @param salted {string}
 * @return {Promise<Boolean>}
 */


async function checkSaltedPassword(pw, salted) {
  return new _bluebird.default((resolve, reject) => {
    _bcrypt.default.compare(pw, salted, (err, res) => {
      err ? reject(err) : resolve(res);
    });
  });
}
/**
 * Create Json Web Token
 *
 * @param secret {string}
 * @param data {object}
 * @return {Promise}
 */


async function encodeJWTToken(secret, data) {
  return new _bluebird.default((resolve, reject) => {
    _jsonwebtoken.default.sign(data, secret, {
      algorithm: 'HS256'
    }, (err, res) => {
      if (err) {
        console.error(64, 'Generating jwt.Token failed!!');
        console.error(err);
        reject(err);
      } else {
        resolve(res);
      }
    });
  });
}
/**
 * Verify JWT token and resolve decoded data.
 *
 * @param secret {string}
 * @param token {string}
 * @return {Promise<Object>}
 */


async function decodeJWTToken(secret, token) {
  return new _bluebird.default((resolve, reject) => {
    _jsonwebtoken.default.verify(token, secret, (err, res) => {
      err ? reject(err) : resolve(res);
    });
  });
}

async function mkdirp(path) {
  return new _bluebird.default((resolve, reject) => {
    (0, _mkdirp.default)(path, err => err ? reject(err) : resolve());
  });
}

const jwtUtil = {
  encode: encodeJWTToken,
  decode: decodeJWTToken
};
exports.jwtUtil = jwtUtil;
const _exports = {
  genSaltedPassword,
  checkSaltedPassword,
  knex,
  Knex: _knex.default,
  jwtUtil,
  moment: _moment.default,
  upsertSQL: _mysql2.upsertSQL,
  mysql: _mysql.default,
  ensure: _odJs.ensure,
  ODApp: _express.default,
  isDeadLockError: _mysql2.isDeadLockError,
  isForeignKeyError: _mysql2.isForeignKeyError,
  isDupKeyError: _mysql2.isDupKeyError,
  getDupKeyErrorIndexName: _mysql2.getDupKeyErrorIndexName,
  handlerMaker: _handler.handlerMaker,
  gmailSender: _sendmail.gmailSender,
  mkdirp,
  SlackNotifier: _slack.SlackNotifier,
  storageDefiner: _definition.storageDefiner,
  AWS: _awsSdk.default
};
var _default = _exports;
exports.default = _default;