"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.copyFile = copyFile;
exports.moveFile = moveFile;
exports.rmrf = rmrf;

var _bluebird = _interopRequireDefault(require("bluebird"));

var _fs = _interopRequireDefault(require("fs"));

var _rimraf = _interopRequireDefault(require("rimraf"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 파일 복사를 수행합니다.
 *
 * @param oldPath {string}
 * @param newPath {string}
 * @return {Promise}
 */
function copyFile(oldPath, newPath) {
  return new _bluebird.default((resolve, reject) => {
    const readStream = _fs.default.createReadStream(oldPath);

    const writeStream = _fs.default.createWriteStream(newPath);

    readStream.on('error', reject);
    writeStream.on('error', reject);
    readStream.on('close', function onClose() {
      resolve();
    });
    readStream.pipe(writeStream);
  });
}
/**
 * 파일을 이동합니다. 만약 실패할 경우 복사를 수행합니다.
 *
 * @param oldPath {string}
 * @param newPath {string}
 * @return {Promise}
 */


function moveFile(oldPath, newPath) {
  return new _bluebird.default((resolve, reject) => {
    _fs.default.rename(oldPath, newPath, function renameCallback(err) {
      if (err) {
        if (err.code === 'EXDEV') {
          resolve(copyFile(oldPath, newPath));
        } else {
          reject(err);
        }
      }

      resolve();
    });
  });
}

async function rmrf(path) {
  return new _bluebird.default((resolve, reject) => {
    (0, _rimraf.default)(path, err => {
      if (err) {
        reject(err);
      } else {
        resolve();
      }
    });
  });
}